/**
 * A class for storing constants that are used throughout the project.
 */
export default class CodegenConstants {
    static readonly API_PACKAGE = "apiPackage";
    static readonly API_PACKAGE_DESC = "package for generated api classes";
    static readonly MODEL_PACKAGE = "modelPackage";
    static readonly MODEL_PACKAGE_DESC = "package for generated models";
    static readonly TEMPLATE_DIR = "templateDir";
    static readonly INVOKER_PACKAGE = "invokerPackage";
    static readonly INVOKER_PACKAGE_DESC = "root package for generated code";
    static readonly GROUP_ID = "groupId";
    static readonly GROUP_ID_DESC = "groupId in generated pom.xml";
    static readonly ARTIFACT_ID = "artifactId";
    static readonly ARTIFACT_ID_DESC = "artifactId in generated pom.xml";
    static readonly ARTIFACT_VERSION = "artifactVersion";
    static readonly ARTIFACT_VERSION_DESC = "artifact version in generated pom.xml";
    static readonly SOURCE_FOLDER = "sourceFolder";
    static readonly SOURCE_FOLDER_DESC = "source folder for generated code";
    static readonly IMPL_FOLDER = "implFolder";
    static readonly IMPL_FOLDER_DESC = "folder for generated implementation code";
    static readonly LOCAL_VARIABLE_PREFIX = "localVariablePrefix";
    static readonly LOCAL_VARIABLE_PREFIX_DESC = "prefix for generated code members and local variables";
    static readonly SERIALIZABLE_MODEL = "serializableModel";
    static readonly SERIALIZABLE_MODEL_DESC = "boolean - toggle \"implements Serializable\" for generated models";
    static readonly SERIALIZE_BIG_DECIMAL_AS_STRING = "bigDecimalAsString";
    static readonly SERIALIZE_BIG_DECIMAL_AS_STRING_DESC = "Treat BigDecimal values as Strings to avoid precision loss.";
    static readonly LIBRARY = "library";
    static readonly LIBRARY_DESC = "library template (sub-template)";
    static readonly SORT_PARAMS_BY_REQUIRED_FLAG = "sortParamsByRequiredFlag";
    static readonly SORT_PARAMS_BY_REQUIRED_FLAG_DESC = "Sort method arguments to place required parameters before optional parameters.";
    static readonly USE_DATETIME_OFFSET = "useDateTimeOffset";
    static readonly USE_DATETIME_OFFSET_DESC = "Use DateTimeOffset to model date-time properties";
    static readonly ENSURE_UNIQUE_PARAMS = "ensureUniqueParams";
    static readonly ENSURE_UNIQUE_PARAMS_DESC = "Whether to ensure parameter names are unique in an operation (rename parameters that are not).";
    static readonly PACKAGE_NAME = "packageName";
    static readonly PACKAGE_VERSION = "packageVersion";
    static readonly PACKAGE_TITLE = "packageTitle";
    static readonly PACKAGE_TITLE_DESC = "Specifies an AssemblyTitle for the .NET Framework global assembly attributes stored in the AssemblyInfo file.";
    static readonly PACKAGE_PRODUCTNAME = "packageProductName";
    static readonly PACKAGE_PRODUCTNAME_DESC = "Specifies an AssemblyProduct for the .NET Framework global assembly attributes stored in the AssemblyInfo file.";
    static readonly PACKAGE_DESCRIPTION = "packageDescription";
    static readonly PACKAGE_DESCRIPTION_DESC = "Specifies a AssemblyDescription for the .NET Framework global assembly attributes stored in the AssemblyInfo file.";
    static readonly PACKAGE_COMPANY = "packageCompany";
    static readonly PACKAGE_COMPANY_DESC = "Specifies an AssemblyCompany for the .NET Framework global assembly attributes stored in the AssemblyInfo file.";
    static readonly PACKAGE_COPYRIGHT = "packageCopyright";
    static readonly PACKAGE_COPYRIGHT_DESC = "Specifies an AssemblyCopyright for the .NET Framework global assembly attributes stored in the AssemblyInfo file.";
    static readonly POD_VERSION = "podVersion";
    static readonly OPTIONAL_METHOD_ARGUMENT = "optionalMethodArgument";
    static readonly OPTIONAL_METHOD_ARGUMENT_DESC = "Optional method argument, e.g. void square(int x=10) (.net 4.0+ only).";
    static readonly OPTIONAL_ASSEMBLY_INFO = "optionalAssemblyInfo";
    static readonly OPTIONAL_ASSEMBLY_INFO_DESC = "Generate AssemblyInfo.cs.";
    static readonly USE_COLLECTION = "useCollection";
    static readonly USE_COLLECTION_DESC = "Deserialize array types to Collection<T> instead of List<T>.";
    static readonly RETURN_ICOLLECTION = "returnICollection";
    static readonly RETURN_ICOLLECTION_DESC = "Return ICollection<T> instead of the concrete type.";
    static readonly OPTIONAL_PROJECT_FILE = "optionalProjectFile";
    static readonly OPTIONAL_PROJECT_FILE_DESC = "Generate {PackageName}.csproj.";
    static readonly OPTIONAL_PROJECT_GUID = "packageGuid";
    static readonly OPTIONAL_PROJECT_GUID_DESC = "The GUID that will be associated with the C# project";
    static readonly MODEL_PROPERTY_NAMING = "modelPropertyNaming";
    static readonly MODEL_PROPERTY_NAMING_DESC = "Naming convention for the property: 'camelCase', 'PascalCase', 'snake_case' and 'original', which keeps the original name";
    static readonly DOTNET_FRAMEWORK = "targetFramework";
    static readonly DOTNET_FRAMEWORK_DESC = "The target .NET framework version.";
    static readonly MODEL_NAME_PREFIX = "modelNamePrefix";
    static readonly MODEL_NAME_PREFIX_DESC = "Prefix that will be prepended to all model names. Default is the empty string.";
    static readonly MODEL_NAME_SUFFIX = "modelNameSuffix";
    static readonly MODEL_NAME_SUFFIX_DESC = "Suffix that will be appended to all model names. Default is the empty string.";
    static readonly OPTIONAL_EMIT_DEFAULT_VALUES = "optionalEmitDefaultValues";
    static readonly OPTIONAL_EMIT_DEFAULT_VALUES_DESC = "Set DataMember's EmitDefaultValue.";
    static readonly GIT_USER_ID = "gitUserId";
    static readonly GIT_USER_ID_DESC = "Git user ID, e.g. swagger-api.";
    static readonly GIT_REPO_ID = "gitRepoId";
    static readonly GIT_REPO_ID_DESC = "Git repo ID, e.g. swagger-codegen.";
    static readonly RELEASE_NOTE = "releaseNote";
    static readonly RELEASE_NOTE_DESC = "Release note, default to 'Minor update'.";
    static readonly HTTP_USER_AGENT = "httpUserAgent";
    static readonly HTTP_USER_AGENT_DESC = "HTTP user agent, e.g. codegen_csharp_api_client, default to 'Swagger-Codegen/{packageVersion}}/{language}'";
    static readonly SUPPORTS_ES6 = "supportsES6";
    static readonly SUPPORTS_ES6_DESC = "Generate code that conforms to ES6.";
    static readonly EXCLUDE_TESTS = "excludeTests";
    static readonly EXCLUDE_TESTS_DESC = "Specifies that no tests are to be generated.";
    static readonly GENERATE_API_TESTS = "generateApiTests";
    static readonly GENERATE_API_TESTS_DESC = "Specifies that api tests are to be generated.";
    static readonly GENERATE_MODEL_TESTS = "generateModelTests";
    static readonly GENERATE_MODEL_TESTS_DESC = "Specifies that model tests are to be generated.";
    static readonly MODEL_PROPERTY_NAMING_TYPE: {
        camelCase: number;
        PascalCase: number;
        snake_case: number;
        original: number;
    };
}
