"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cleanGenerated = exports.regenerateCode = exports.generateApi = void 0;
const generateProject_1 = __importStar(require("./generateProject"));
const normalizeConfig_1 = __importDefault(require("./normalizeConfig"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const semver_1 = __importDefault(require("semver"));
const ern_core_1 = require("ern-core");
const inquirer_1 = __importDefault(require("inquirer"));
const { execp } = ern_core_1.childProcess;
/**
 * ==============================================================================
 * Main entry point
 * ==============================================================================
 *
 * Refer to normalizeConfig function doc for the list of options
 */
async function generateApi(options) {
    const config = (0, normalizeConfig_1.default)(options);
    const outFolder = path_1.default.join(process.cwd(), config.packageName);
    if (fs_extra_1.default.pathExistsSync(outFolder)) {
        ern_core_1.log.error(`${outFolder} directory already exists`);
        process.exit(1);
    }
    // Create output folder
    ern_core_1.shell.mkdir(outFolder);
    await (0, generateProject_1.default)(config, outFolder);
    ern_core_1.log.info(`==  Project was generated in ${outFolder}`);
}
exports.generateApi = generateApi;
/**
 * @param options
 * @returns {Promise.<void>}
 */
async function regenerateCode(options = {}) {
    var _a, _b, _c, _d;
    const pkg = await validateApiNameAndGetPackageJson('This is not a properly named API directory. Naming convention is react-native-{name}-api');
    const curVersion = pkg.version || '1.0.0';
    const pkgName = pkg.name;
    let newPluginVer;
    if (options.skipVersion) {
        newPluginVer = curVersion;
    }
    else {
        newPluginVer = semver_1.default.inc(curVersion, 'minor');
        const { confirmPluginVer } = await inquirer_1.default.prompt([
            {
                message: `Would you like to bump the plugin version from [${pkgName}@${curVersion}] to [${pkgName}@${newPluginVer}]?`,
                name: 'confirmPluginVer',
                type: 'confirm',
            },
        ]);
        if (!confirmPluginVer) {
            newPluginVer = await _promptForPluginVersion(curVersion);
        }
    }
    await _checkDependencyVersion(pkg, options.targetDependencies || []);
    const isNewVersion = semver_1.default.lt(curVersion, newPluginVer);
    const extra = (_b = (_a = pkg === null || pkg === void 0 ? void 0 : pkg.ern) === null || _a === void 0 ? void 0 : _a.message) !== null && _b !== void 0 ? _b : {};
    const config = (0, normalizeConfig_1.default)({
        apiAuthor: pkg.author,
        apiDescription: pkg.description,
        apiVersion: isNewVersion ? newPluginVer : pkg.version,
        artifactId: pkg.artifactId,
        bridgeVersion: options.bridgeVersion ||
            pkg.peerDependencies['react-native-electrode-bridge'],
        packageName: pkgName,
        ...extra,
        ...options,
        name: (_d = (_c = extra === null || extra === void 0 ? void 0 : extra.apiName) !== null && _c !== void 0 ? _c : extra === null || extra === void 0 ? void 0 : extra.moduleName) !== null && _d !== void 0 ? _d : pkgName,
    });
    await cleanGenerated();
    // Regenerate package.json
    await fs_extra_1.default.writeFile(path_1.default.join(process.cwd(), 'package.json'), (0, generateProject_1.generatePackageJson)(config));
    await (0, generateProject_1.generateSwagger)(config, process.cwd());
    ern_core_1.log.info('== API generation complete.');
    isNewVersion
        ? await publish(await readPackage())
        : ern_core_1.log.info('Done. Remember to publish a new version if needed.');
}
exports.regenerateCode = regenerateCode;
async function cleanGenerated(outFolder = process.cwd()) {
    const pkg = await validateApiNameAndGetPackageJson('This is not a properly named API directory. Naming convention is react-native-{name}-api');
    ern_core_1.shell.rm('-rf', path_1.default.join(outFolder, 'IOS')); // APIs generated with ERN <= 0.49
    ern_core_1.shell.rm('-rf', path_1.default.join(outFolder, 'android'));
    ern_core_1.shell.rm('-rf', path_1.default.join(outFolder, 'ios'));
    ern_core_1.shell.rm('-rf', path_1.default.join(outFolder, 'javascript'));
    ern_core_1.shell.rm('-rf', path_1.default.join(outFolder, 'package.json'));
    return pkg;
}
exports.cleanGenerated = cleanGenerated;
async function validateApiNameAndGetPackageJson(message) {
    const pkg = await readPackage();
    if (!(await ern_core_1.utils.isDependencyApi(pkg.name))) {
        throw new Error(message);
    }
    return pkg;
}
async function readPackage() {
    return fs_extra_1.default.readJson(path_1.default.join(process.cwd(), 'package.json'));
}
const nextVersion = (curVersion, userPluginVer) => {
    switch (userPluginVer.toLowerCase()) {
        case 'same':
        case 'no':
        case 'q':
        case 'quit':
        case 'n':
            return curVersion;
        default: {
            try {
                // If valid return
                if (semver_1.default.valid(userPluginVer) != null) {
                    return userPluginVer;
                }
                const ret = semver_1.default.inc(curVersion, userPluginVer);
                if (ret) {
                    return ret;
                }
            }
            catch (e) {
                ern_core_1.log.info(`Not a valid version: ${userPluginVer}`);
            }
        }
    }
};
async function _promptForPluginVersion(curVersion) {
    const { userPluginVer } = await inquirer_1.default.prompt([
        {
            message: `Current Plugin Version is ${curVersion}. Type the new plugin version (<newversion> | major | minor | patch | premajor | preminor | prepatch | prerelease | same)?`,
            name: 'userPluginVer',
            type: 'input',
        },
    ]);
    const ret = nextVersion(curVersion, userPluginVer);
    if (ret == null) {
        ern_core_1.log.info('Enter a valid version. For more details visit https://github.com/npm/node-semver');
        return _promptForPluginVersion(curVersion);
    }
    return ret;
}
async function _checkDependencyVersion(pkg, targetDependencies) {
    const pluginDependency = pkg.peerDependencies || {};
    const targetNativeDependenciesMap = _constructTargetNativeDependenciesMap(targetDependencies);
    for (const key of Object.keys(pluginDependency)) {
        if (targetNativeDependenciesMap.has(key) &&
            pluginDependency[key] !== targetNativeDependenciesMap.get(key)) {
            const answer = await _promptForMissMatchOfSupportedPlugins(targetNativeDependenciesMap.get(key), key);
            pluginDependency[key] = answer.userPluginVer
                ? answer.userPluginVer
                : targetNativeDependenciesMap.get(key);
        }
    }
}
function _constructTargetNativeDependenciesMap(targetDependencies) {
    return new Map(targetDependencies.map((curVal) => {
        const dependencyString = curVal.toString();
        const idx = dependencyString.lastIndexOf('@'); // logic for scoped dependency
        return [
            dependencyString.substring(0, idx),
            dependencyString.substring(idx + 1),
        ];
    }));
}
function _promptForMissMatchOfSupportedPlugins(curVersion, pluginName) {
    return inquirer_1.default.prompt([
        {
            message: `Type the new plugin version for ${pluginName}. Press Enter to use the default '${curVersion}'.`,
            name: 'userPluginVer',
            type: 'input',
        },
    ]);
}
async function publish({ version }) {
    const answers = await inquirer_1.default.prompt([
        {
            message: `Would you like to npm publish version [${version}] of this API?`,
            name: 'confirmNpmPublish',
            type: 'confirm',
        },
    ]);
    if (answers.confirmNpmPublish) {
        await npmPublish();
    }
}
async function npmPublish() {
    return execp('npm publish');
}
exports.default = {
    cleanGenerated,
    generateApi,
    regenerateCode,
};
//# sourceMappingURL=apigen.js.map