"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateGitignore = exports.generateInitialSchema = exports.generatePackageJson = exports.generateSwagger = exports.GENERATE = void 0;
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const ern_core_1 = require("ern-core");
const index_1 = require("./index");
exports.GENERATE = [
    ['android', 'ERNAndroid'],
    ['ios', 'ERNSwift'],
    ['javascript', 'ERNES6'],
];
async function generateSwagger({ apiSchemaPath = 'schema.json', name, namespace = '', ...optional }, outFolder) {
    const inputSpec = path_1.default.resolve(outFolder, apiSchemaPath);
    const groupId = namespace.toLowerCase().replace(/[^a-z0-9._]/g, '');
    const shared = {
        apiPackage: `${groupId}.api`,
        description: optional.apiDescription,
        groupId,
        inputSpec,
        modelPackage: `${groupId}.model`,
        projectVersion: optional.apiVersion,
        version: optional.apiVersion,
        ...optional,
    };
    for (const [projectName, lang] of exports.GENERATE) {
        const cc = new index_1.CodegenConfigurator({
            ...shared,
            lang,
            outputDir: outFolder + '/' + projectName,
            projectName,
        });
        const opts = await cc.toClientOptInput();
        new index_1.DefaultGenerator().opts(opts).generate();
    }
}
exports.generateSwagger = generateSwagger;
function generatePackageJson({ npmScope, reactNativeVersion, apiVersion, apiDescription, apiAuthor, apiLicense, bridgeVersion, packageName, ...conf }) {
    // Reset the apiSchemaPath to schema.json
    // if --schemaPath option is used to create the Api
    const options = { ...conf, apiSchemaPath: 'schema.json' };
    const dependencies = bridgeVersion
        ? {
            'react-native-electrode-bridge': `^${bridgeVersion}`,
        }
        : undefined;
    // tslint:disable:object-literal-sort-keys
    return JSON.stringify({
        name: npmScope ? `@${npmScope}/${packageName}` : packageName,
        version: apiVersion !== null && apiVersion !== void 0 ? apiVersion : '1.0.0',
        description: apiDescription,
        main: 'javascript/src/index.js',
        keywords: [`${ern_core_1.ModuleTypes.API}`],
        author: apiAuthor,
        license: apiLicense,
        dependencies,
        ern: {
            message: options,
            moduleType: `${ern_core_1.ModuleTypes.API}`,
        },
    }, null, 2).concat('\n');
}
exports.generatePackageJson = generatePackageJson;
async function generateInitialSchema({ apiSchemaPath, }) {
    const pathToSchemaFile = apiSchemaPath && fs_extra_1.default.existsSync(apiSchemaPath)
        ? apiSchemaPath
        : path_1.default.join(__dirname, '..', 'initialApiSchema.json');
    return fs_extra_1.default.readFile(pathToSchemaFile);
}
exports.generateInitialSchema = generateInitialSchema;
function generateGitignore() {
    return `.DS_Store
.idea/
node_modules/
`;
}
exports.generateGitignore = generateGitignore;
async function generateProject(config = {}, outFolder) {
    await fs_extra_1.default.writeFile(path_1.default.join(outFolder, '.gitignore'), generateGitignore());
    await fs_extra_1.default.writeFile(path_1.default.join(outFolder, 'package.json'), generatePackageJson(config));
    await fs_extra_1.default.writeFile(path_1.default.join(outFolder, 'schema.json'), await generateInitialSchema(config));
    await generateSwagger(config, outFolder);
}
exports.default = generateProject;
//# sourceMappingURL=generateProject.js.map