"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* tslint:disable:variable-name */
const mustache_1 = __importDefault(require("mustache"));
const Json_1 = __importDefault(require("./Json"));
const ern_core_1 = require("ern-core");
const MustacheWriter_1 = require("./MustacheWriter");
const isIterable_1 = require("./isIterable");
function truthy(val) {
    if (val == null || val.length === 0 || val === false) {
        return false;
    }
    return true;
}
function isInt(val) {
    return /^\d+?$/.test(val);
}
class MyContext extends mustache_1.default.Context {
    constructor(value, parent, first, last) {
        super(value, parent);
        this._first = first;
        this._last = last;
    }
    lookup(name) {
        if (/-?last/.test(name)) {
            return this._last;
        }
        if (/-?first/.test(name)) {
            return this._first;
        }
        // match the dot, unless it is followed only by a number, than we guess its an array and
        // we will let the natural path happen.
        if (/.+?\.(?!\d+?$).+?$/.test(name)) {
            const filters = name.split(/(\-|\+|\.)/);
            const arr = super.lookup(filters.shift());
            if (!(0, isIterable_1.isIterable)(arr)) {
                return arr;
            }
            // Using a Set to prevent duplicates, this may be a mistake.
            const ret = new Set();
            FILTER: for (let v of arr) {
                for (let i = 0, l = filters.length; i < l; i += 2) {
                    const pm = filters[i];
                    const prop = filters[i + 1];
                    if (pm === '+') {
                        if (!truthy(v[prop])) {
                            continue FILTER;
                        }
                    }
                    else if (pm === '-') {
                        if (truthy(v[prop])) {
                            continue FILTER;
                        }
                    }
                    else if (pm === '.' && prop) {
                        if (v != null) {
                            v = v[prop];
                        }
                    }
                }
                ret.add(v);
            }
            return Array.from(ret);
        }
        return super.lookup(name);
    }
    push(view, first, last) {
        return new MyContext(view, this, first, last);
    }
}
exports.default = {
    compiler() {
        let defValue;
        const partials = {};
        let partialProxy;
        const cret = {
            withLoader(_loader) {
                const handler = {
                    get(target, name) {
                        if (name in target) {
                            return target[name];
                        }
                        return (target[name] = _loader.getTemplate(name));
                    },
                };
                partialProxy = (name) => {
                    return _loader.getTemplate(name);
                };
                return cret;
            },
            defaultValue(def) {
                defValue = def;
                return cret;
            },
            compile(template, file) {
                const writer = new MustacheWriter_1.MustacheWriter();
                return {
                    execute(data) {
                        try {
                            data = JSON.parse(Json_1.default.pretty(data));
                            return writer.render(template, new MyContext(data), partialProxy);
                        }
                        catch (e) {
                            ern_core_1.log.trace(e);
                            throw new Error(`Error invoking template ${file}`);
                        }
                    },
                };
            },
        };
        return cret;
    },
};
//# sourceMappingURL=Mustache.js.map