"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SEARCH_PATH = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const ern_core_1 = require("ern-core");
const File_1 = __importDefault(require("./File"));
const StringUtils_1 = require("./StringUtils");
const tryNewRequire = (mod) => {
    try {
        const Clz = require(mod).default;
        return new Clz();
    }
    catch (e) {
        ern_core_1.log.warn(`could not require ${mod}. error: ${e}`);
    }
};
exports.SEARCH_PATH = [path_1.default.join(__dirname, '..', '..', 'resources')];
exports.default = {
    load(className) {
        const ret = [];
        const lines = [];
        for (const searchPath of exports.SEARCH_PATH) {
            const meta = new File_1.default(searchPath, 'META-INF', 'services', className);
            if (!meta.exists()) {
                continue;
            }
            try {
                lines.push(...fs_1.default.readFileSync(meta.getPath(), 'utf8').split(/\r?\n/));
            }
            catch (e) {
                ern_core_1.log.warn(`Error loading ${className}. error: ${e}`);
                return ret;
            }
        }
        for (const line of lines) {
            const [mod, comment] = line.split('#', 2);
            if ((0, StringUtils_1.isEmpty)(mod)) {
                continue;
            }
            const conf = tryNewRequire(path_1.default.join(__dirname, '..', ...mod.split('.')));
            if (conf) {
                ret.push(conf);
            }
        }
        return ret;
    },
};
//# sourceMappingURL=ServiceLoader.js.map