"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractSecuritySchemeDefinition_1 = __importDefault(require("./AbstractSecuritySchemeDefinition"));
const javaUtil_1 = require("../../java/javaUtil");
class OAuth2Definition extends AbstractSecuritySchemeDefinition_1.default {
    constructor() {
        super(...arguments);
        this.type = OAuth2Definition.TYPE;
    }
    implicit(authorizationUrl) {
        this.setAuthorizationUrl(authorizationUrl);
        this.setFlow('implicit');
        return this;
    }
    password(tokenUrl) {
        this.setTokenUrl(tokenUrl);
        this.setFlow('password');
        return this;
    }
    application(tokenUrl) {
        this.setTokenUrl(tokenUrl);
        this.setFlow('application');
        return this;
    }
    accessCode(authorizationUrl, tokenUrl) {
        this.setTokenUrl(tokenUrl);
        this.setAuthorizationUrl(authorizationUrl);
        this.setFlow('accessCode');
        return this;
    }
    scope(name, description) {
        this.addScope(name, description);
        return this;
    }
    getAuthorizationUrl() {
        return this.authorizationUrl;
    }
    setAuthorizationUrl(authorizationUrl) {
        this.authorizationUrl = authorizationUrl;
    }
    getTokenUrl() {
        return this.tokenUrl;
    }
    setTokenUrl(tokenUrl) {
        this.tokenUrl = tokenUrl;
    }
    getFlow() {
        return this.flow;
    }
    setFlow(flow) {
        this.flow = flow;
    }
    getScopes() {
        return this.scopes;
    }
    setScopes(scopes) {
        this.scopes = (0, javaUtil_1.asMap)(scopes);
    }
    addScope(name, description) {
        if (this.scopes == null) {
            this.scopes = (0, javaUtil_1.newHashMap)();
        }
        this.scopes.put(name, description);
    }
    toJSON() {
        return Object.assign(super.toJSON(), {
            authorizationUrl: this.authorizationUrl,
            flow: this.flow,
            scopes: this.scopes,
            tokenUrl: this.tokenUrl,
            type: this.type,
        });
    }
}
exports.default = OAuth2Definition;
OAuth2Definition.TYPE = 'oauth2';
//# sourceMappingURL=OAuth2Definition.js.map