"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectPropertyBase = void 0;
/* tslint:disable:variable-name */
const Property_1 = require("./Property");
const factory_1 = __importDefault(require("../factory"));
const javaUtil_1 = require("../../java/javaUtil");
const beanUtils_1 = require("../../java/beanUtils");
function objectToPropertyMap(obj) {
    return (0, javaUtil_1.newHashMap)(...Object.keys(obj).map((key) => [key, (0, factory_1.default)(obj[key], obj)]));
}
class ObjectPropertyBase extends Property_1.Property {
    constructor(obj, parent) {
        super(obj, parent);
    }
    getAdditionalProperties() {
        return this._additionalProperties;
    }
    setAdditionalProperties(properties) {
        if (!properties) {
            this._additionalProperties = null;
        }
        // @todo the ref support here is only dangerously supported
        // { $ref: '#/definition/ModelName }
        if (properties.type || properties.$ref || properties.genericRef) {
            this._additionalProperties = (0, factory_1.default)(properties);
        }
        else {
            this._additionalProperties = objectToPropertyMap(properties);
        }
    }
    setProperties(properties) {
        this.properties = objectToPropertyMap(properties);
    }
    getRequiredProperties() {
        const required = [];
        for (const [key, prop] of this.properties) {
            if (prop.required) {
                required.push(key);
            }
        }
        return required;
    }
    setRequiredProperties(required) {
        for (const [key, prop] of this.properties) {
            const isRequired = required ? required.indexOf(key) !== -1 : false;
            if ((0, beanUtils_1.has)(prop, 'required') || isRequired) {
                prop.required = isRequired;
            }
        }
    }
}
exports.ObjectPropertyBase = ObjectPropertyBase;
ObjectPropertyBase.allowedProps = [
    ...Property_1.Property.allowedProps,
    'additionalProperties',
    'required',
    'minProperties',
    'maxProperties',
    'anyOf',
    'allOf',
    'readOnly',
    'vendorExtensions',
    'xml',
];
//# sourceMappingURL=ObjectPropertyBase.js.map